/*
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Copyright (c) 2007 Andrei Borovsky <anb@symmetrica.net>
 */
 
#include <wx/wx.h>
#include "include/mainframe.h"

#define ID_START wxID_HIGHEST + 1
#define ID_ENTER wxID_HIGHEST + 2
#define ID_CONNECTION wxID_HIGHEST + 3



MainFrame::MainFrame(wxWindow* parent, wxWindowID id, const wxString& title, const wxPoint& pos, const wxSize& size, long style, const wxString& name):wxFrame(parent, id, title, pos, size, style, name)
{
	vbox = new wxBoxSizer(wxVERTICAL);
	hbox = new wxBoxSizer(wxHORIZONTAL);
	commandEdit = new wxTextCtrl(this, ID_ENTER, "PROTOCOL 6", wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER);
	hbox->Add(commandEdit, 1, wxEXPAND|wxALL, 2);
	commandButton = new wxButton(this, ID_START, "Send");
	hbox->Add(commandButton, 0, wxALIGN_LEFT|wxALL, 2);
	vbox->Add(hbox, 0, wxEXPAND);
	logViewer = new wxTextCtrl(this, wxID_ANY, "", wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE|wxTE_READONLY|wxTE_RICH);
	vbox->Add(logViewer, 1, wxEXPAND|wxALIGN_CENTER|wxALL, 2);
	SetSizer(vbox);
	vbox->Fit(this);

	connection = new wxDBusConnection(ID_CONNECTION, this, false);
	Connect(ID_CONNECTION, wxEVT_DBUS_SIGNAL,
		wxDBusConnectionEventHandler(MainFrame::OnSignal));
	Connect(ID_CONNECTION, wxEVT_DBUS_NOTIFICATION,
		wxDBusConnectionEventHandler(MainFrame::OnNotification));
    connection->RegisterObjectPath("/com/Skype/Client");
 	Connect(ID_START, wxEVT_COMMAND_BUTTON_CLICKED,
		wxCommandEventHandler(MainFrame::OnCommand));
	Connect(ID_ENTER, wxEVT_COMMAND_TEXT_ENTER,
		wxCommandEventHandler(MainFrame::OnCommand));
	
	SkypeCommand("NAME wxWidgets_SKYPE_Monitor");
}

void MainFrame::OnCommand(wxCommandEvent& event)
{
	SkypeCommand(commandEdit->GetLineText(0));
}

void MainFrame::OnSignal(wxDBusConnectionEvent& event)
{
	wxDBusMessage * msg = wxDBusMessage::ExtractFromEvent(&event);
	const char * resp = msg->GetString();
	printf("Signal: %s\n", resp);
	delete msg;
}

void MainFrame::OnNotification(wxDBusConnectionEvent& event)
{
	wxDBusMessage * msg = wxDBusMessage::ExtractFromEvent(&event);
	const char * resp;
	/*
	  When the Skype client wants to tell us something it calls 
	  the Notify method of the /com/Skype/Client class we have registered.
	  This method call message is the only type of message we can get here,
	  but we will still check the message type.
	*/
	
	if ((msg->GetType() == DBUS_MESSAGE_TYPE_METHOD_CALL) &&
		(wxString(msg->GetMember()) == wxString("Notify"))) {
		// Yes, it is a method call, and it calls the Notify method.
		// Start parsing arguments.
		if (msg->MoveToNextArg())
		{
			msg->GetArgValue(&resp);
			logViewer->SetDefaultStyle(wxTextAttr(*wxGREEN));
			logViewer->AppendText("NOTIFICATION: "); 
			logViewer->AppendText(resp);
			logViewer->AppendText("\n");
		}
	}
	delete msg;
}

void MainFrame::SkypeCommand(const wxString & command)
{
	logViewer->SetDefaultStyle(wxTextAttr(*wxBLUE));
	logViewer->AppendText("COMMAND: "); 
	logViewer->AppendText(command);
	logViewer->AppendText("\n");
	wxDBusMethodCall * invoke = new wxDBusMethodCall(
		"com.Skype.API",
		"/com/Skype",
		"com.Skype.API",
		"Invoke");
	invoke->AddString(command);
	wxDBusMessage * response = invoke->Call(connection, 10000);
	// Now check if the call was successful
	if (response == NULL) {
		wxMessageDialog * errorDlg = new wxMessageDialog(this, "Sending command \"" + command +
		 "\" failed.\n Either Skype instance is not running or D-Bus support is turned off.",
		"Communication Error", wxICON_ERROR|wxOK);
		errorDlg->ShowModal();
		delete errorDlg;
	} else	{
		logViewer->AppendText("RESPONSE: "); 
		logViewer->AppendText(response->GetString());
		logViewer->AppendText("\n");
		delete response;
		delete invoke;
	}
}

MainFrame::~MainFrame()
{
	delete connection;
}
